#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "cool.h"

static double *ztab, *ttab, *ctab;
static int NT, NZ;

int read_ct_plewa(char *ctfile) {
    FILE *f;
    int i, j, k;

    NZ = COOL_NZ_PLEWA;
    NT = COOL_NT_PLEWA;

    // read 1st line: set of metalicities
    ztab = malloc(NZ*sizeof(double));
    ttab = malloc(NT*sizeof(double));
    ctab = malloc(NZ*NT*sizeof(double));
    f = fopen(ctfile, "r");
    for (i = 0; i < NZ; i++) {
        fscanf(f, "%lf", &ztab[i]);
    }

    for (i = 0; i < NT; i++) {
        fscanf(f, "%d %lf", &k, &ttab[i]);
        //printf("%d - %d %10.3e\n", i, k, ttab[i]);
        for (j = 0; j < NZ; j++) {
            fscanf(f, "%lf", &ctab[j*NT+i]);
            //printf("        %10.3e\n", ctab[j*COOL_NT+i]);
        }
    }

    fclose(f);
    return 0;
}

int read_ct_sd93() {
    FILE *f;
    int i, j, k;
    double temp, cf;

    NZ = COOL_NZ_SD93;
    NT = COOL_NT_SD93;

    ztab = malloc(NZ*sizeof(double));
    ttab = malloc(NT*sizeof(double));
    ctab = malloc(NZ*NT*sizeof(double));
    f = fopen("sd93-cf", "r");
    ztab[0] = 1;

    for (i = 0; i < NT; i++) {
        fscanf(f, "%lf %*lf %*lf %*lf %*lf %lf %*[^\n]\n", &temp, &cf);
        ttab[i] = pow(10, temp);
        ctab[i] = pow(10, cf);
        //printf("%10.3e %10.3e\n", ttab[i], ctab[i]);
    }

    fclose(f);
    return 0;
}

void free_ct()
{
    free(ztab);
    free(ttab);
    free(ctab);
}



double cool(double t, double z)
{
    int i, j;
    double p, q;

    i = intrvl(t, ttab, NT);
    p = (t-ttab[i])/(ttab[i+1]-ttab[i]);

    if (NZ == 1) {
        //printf("t,cf: %14.7e %14.7e\n", t, ((1.-p)*ctab[i] + p*ctab[i+1]));
        return ((1.-p)*ctab[i] + p*ctab[i+1]);
    } else {
        j = intrvl(z, ztab, NZ);
        q = (z-ztab[j])/(ztab[j+1]-ztab[j]);
        return (1.-q)*((1.-p)*ctab[j*NT+i]     + p*ctab[j*NT+i+1])
        +          q *((1.-p)*ctab[(j+1)*NT+i] + p*ctab[(j+1)*NT+i+1]);
    }

    //print "i, j: ", i, j
    //print "p,q: ", p, q

}


double dcool(double t, double z)
{
    int i, j;
    double p, q;

    i = intrvl(t, ttab, NT);
    //print "i, j: ", i, j

    if (NZ == 1) {
        return (ctab[i] - ctab[i+1]) / (ttab[i+1]-ttab[i]);
    } else {
        j = intrvl(z, ztab, NZ);
        q = (z-ztab[j])/(ztab[j+1]-ztab[j]);

        return ((1.-q)*(ctab[j*NT+i] - ctab[j*NT+i+1]) 
         + q*(ctab[(j+1)*NT+i] - ctab[(j+1)*NT+i+1]))
         / (ttab[i+1]-ttab[i]);
    }

}

int intrvl(double x, double *arr, int n)
{
    int i, ih, il;
    
    if (x < arr[0]) return 0;
    else if (x > arr[n-1]) return n-1;
    else {
        il = 0;
        ih = n-1;
        while ((ih-il) > 1) {
            i = (ih+il)/2;
            if (x < arr[i]) ih = i;
            else il = i;
        }
    }
    return il;
}



