#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "windlib.h"
#include "cool.h"



int main()
{
    t_clpar clpar;
    t_eos eos;
    t_numpar numpar;
    double gamma, mu_a, mu_i, metal;
    double qe, qm;
    char cftype[32];
    int i;
    t_solvec *sol;

    double Tst, Rst = 0, rhost, Lc, r, Ld;
    double Ta, Tb, Tc, nst;
    double Rson_test;

    read_winit(&clpar, &eos, &numpar, cftype);
    printf("all read\n");

    if (!strcmp(cftype, "plewa")) {
        //printf("reading plewa cf\n");
        read_ct_plewa("/home/richard/codes/windcalc/cooltab");
    } else if (!strcmp(cftype, "sd93")) {
        //printf("reading sd93 cf\n");
        read_ct_sd93();
    }

    //printf("%10.3e  %10.3e  %10.3e\n", cool(10,1), cool(1000,1), cool(1e7,1));

    printf("---- INPUT PARAMETERS ----\n");
    printf("LSC       = %12.5e erg/s\n", clpar.dE);
    printf("RSC       = %12.5e pc\n", clpar.Rsc/cmpc);
    printf("vinf_adia = %12.5e km/s\n", clpar.vinf/1e5);
    printf("eta       = % g\n", clpar.eta);
    printf("Z         = % g\n", eos.metal);
    printf("gamma     = % g\n", eos.gamma);
    printf("mu_a      = % g\n", eos.mu_a);
    printf("mu_i      = % g\n", eos.mu_i);

    printf("\n");
    printf("---- MASS AND ENERGY INSERTION ----\n");
    clpar.dM = vinf2dM(clpar.dE, clpar.vinf, clpar.eta);
    qe = get_qe(clpar.dE, clpar.Rsc, clpar.eta);
    qm = get_qm(clpar.dM, clpar.Rsc);
    printf("dM = %12.5e g/s = % g MSun/yr\n", clpar.dM, clpar.dM/MSun*syr);
    printf("qm = %12.5e\n", qm);
    printf("qe = %12.5e\n", qe);

    printf("\n");
    printf("---- PARAMETERS OF BIMODAL SOLUTION ----\n");
    Ta = T_c_adia(qe, qm, eos);
    printf("T_c,adia  = %12.5e K\n", Ta);
    Tb = get_T_Pmax(qe, qm, eos, numpar.NT);
    printf("T_Pmax    = %12.5e K (same as T_st for R_st > 0)\n", Tb);

    //printf("Rsonic\n");
    //Rsonic(Ta, 0.0, clpar, eos, Rmax, Nr);

    //printf("rk4_integrate\n");
    //rk4_integrate(Ta, 0.0, clpar, eos, Rmax, Nr);

        //Rst = find_R_st(Tb, clpar, eos, Rmax, rk4tol, Nr, NT);
        //printf("R_st      = % g pc \n", Rst/cmpc);

    Lc = find_Lcrit_new(Tb, clpar, eos, numpar);
    //Ld = find_Lcrit(Lc/100, Lc*100, Tb, Rsc, vinf, eta, gamma, mu_a, mu_i, metal, Rmax, Nr, NT);

    //printf("Lcrit12 = %15.8e %15.8e\n", Lc, Ld);
    if (clpar.dE > Lc) {
        Rst = find_R_st(Tb, clpar, eos, numpar);
        Tc = Tb;
    } else {
        Rst = 0;
        Tc = find_T_st(Rst, Tb, clpar, eos, numpar);
    }
    nst = n_st(Tc, qe, qm, eos);
    printf("T_st      = %12.5e K\n", Tc);
    printf("R_st      = % g pc \n", Rst/cmpc);
    printf("n_st      = %12.5e g/cm^3\n", nst);
    printf("P_st      = %12.5e dyne/cm^2\n", nst*kB*Tc);
    printf("L_crit    = %12.5e erg/s\n", Lc);

    Rson_test = 0;
    sol = rk4_integrate(Tc, Rst, clpar, eos, numpar, &Rson_test);
    printf("\n");
    printf("---- WIND SOLUTION ----\n");
    printf(" r               rho             u               P\n");
    for (i = 0; i < numpar.Nrmin; i++) {
        if (sol[i].r < 0) break;
        printf("%15.8e %15.8e %15.8e %15.8e %15.8e %15.8e\n", sol[i].r, sol[i].rho, sol[i].u, sol[i].P, eos.mu_a*mH*sol[i].P/sol[i].rho/kB, sqrt(eos.gamma*sol[i].P/sol[i].rho));
    }
    if (Rson_test == -2) printf("Integration not successful.\n");

    free_ct();
    free(sol);
}





