#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "windlib.h"
#include "cool.h"

//#define DEBUG
#define alpha 0.28
#define TINY 1e-99
#define NMAXSTEPS 10000

#define MIN(X,Y) ((X) < (Y) ? (X) : (Y))
#define MAX(X,Y) ((X) > (Y) ? (X) : (Y))

double B(double gamma)
{
    return (pow((gamma-1.)/(gamma+1.), 0.5) * pow((gamma+1.)/(6*gamma+2.), (3*gamma+1.)/(5*gamma+1.)));
}

double get_qe(double dE, double Rsc, double eta)
{
    return 3*eta*dE/(4*pi*pow(Rsc,3));
}

double get_qm(double dM, double Rsc)
{
    return 3*dM/(4*pi*pow(Rsc, 3));
}

double rho_c_adia(double dM, double Rsc, double vinf, double gamma)
{
    return dM/(4*pi*B(gamma)*Rsc*Rsc*vinf);
}

double P_c_adia(double dM, double Rsc, double vinf, double gamma)
{
    return (gamma-1.)/(2*gamma) * (dM*vinf)/(4*pi*B(gamma)*Rsc*Rsc);
}

double T_c_adia(double qe, double qm, t_eos eos)
{
    return (eos.gamma-1.)/eos.gamma * (eos.mu_a*mH)/kB * qe/qm;
}

double cs2(double T, t_eos eos)
{
    return (eos.gamma*kB*T/eos.mu_a/mH);
}

double dM2vinf(double dE, double dM, double eta)
{
    //return sqrt(2*eta*dE/dM);
    return sqrt(2*dE/dM);
}

double vinf2dM(double dE, double vinf, double eta)
{
    //return 2*eta*dE/(vinf*vinf);
    return 2*dE/(vinf*vinf);
}

double n_st(double T_st, double qe, double qm, t_eos eos)
{
    double D;

    D = (qe - qm*cs2(T_st, eos)/(eos.gamma-1.))/cool(T_st,eos.metal);
    if (D > 0) return sqrt(D);
    else return -1.0;
}

double P_st(double T_st, double qe, double qm, t_eos eos)
{
    double nst;

    nst = n_st(T_st, qe, qm, eos);
    if (nst < 0) return -1;
    else return (nst*eos.mu_i*mH)*kB*T_st / (eos.mu_a*mH);
}

double get_T_Pmax(double qe, double qm, t_eos eos, double NT)
{
    double Tmin, Tmax;
    double Pmax, T, TPmax, nc, Pc, Teps, Tfac;
    double T0, T1, T2, T3, n1, n2, P1, P2, C, R;
    int k;

    Tmax = T_c_adia(qe, qm, eos);
    Tmin = 10.;
    // at first fing the approximate position of the global maximum
    // by testing 1000 points distributed logarithmicaly between Tmin and Tmax
    Tfac = 0.999;
    Pmax = 0;
    T = Tmax;
    TPmax = T;
    while (T > Tmin) {
        nc = n_st(T, qe, qm, eos);
        //Pc = (nc*mu_i*mH)*kB*T / (mu_a*mH);
        Pc = P_st(T, qe, qm, eos);
        if (Pc > Pmax) {
            Pmax = Pc;
            TPmax = T;
        //} else {
        //    return TPmax; // we search for the first local maxima
        }
        T *= Tfac;
        //printf("TPm: %14.7e %14.7e %14.7e %14.7e %14.7e\n", T, nc, Pc, Pmax, TPmax);
    }
    
    // refine the position of the maximum be golden section search 
    C = (3-sqrt(5))/2;
    R = 1-C;
    Teps = Tmin/NT;

    T0 = TPmax*0.999;
    T3 = TPmax/0.999;
    T2 = TPmax;
    T1 = TPmax - C*(T2-T0);
    P1 = P_st(T1, qe, qm, eos);
    P2 = P_st(T2, qe, qm, eos);
    k = 0;
    while (fabs(T3-T0) > Teps) {

        if (P2 > P1) {
            T0 = T1;
            T1 = T2;
            T2 = R*T1 + C*T3;   // T2 = T1+c*(T3-T1)
            P1 = P2;
            P2 = P_st(T2, qe, qm, eos);
        } else {
            T3 = T2;
            T2 = T1;
            T1 = R*T2 + C*T0;   // T1 = T2+c*(T0-T2)
            P2 = P1;
            P1 = P_st(T1, qe, qm, eos);
        }
        //printf("k=%4d, |a-b|=%12.5e, T12,P12 = %15.8e %15.8e, %15.8e %15.8e\n", k, fabs(T3-T0), T1, T2, P1, P2);
        k++;
    }

    if (P1 > P2) {
        TPmax = T1;
        //Pmax = P1;
    } else {
        TPmax = T2;
        //Pmax = P2;
    }

    return TPmax;
}
    
double dudr7(double r, double u, double T, double rho, double R_st, double qe, double qm, t_eos eos)
{
    double Q, rstfac;

    Q = pow((rho/(mH*eos.mu_i)), 2) * cool(T, eos.metal);
    if (R_st == 0) 
        rstfac = 1;
    else 
        rstfac = (1-pow(R_st/r, 3));
    printf("dudr7: %12.5e %12.5e\n", ((eos.gamma-1.)*(qe-Q)+qm*(0.5*(eos.gamma+1.)*u*u - 2.*rstfac*cs2(T, eos)/3.)) , (rho*(cs2(T, eos) - u*u)));
    return ((eos.gamma-1.)*(qe-Q)+qm*(0.5*(eos.gamma+1.)*u*u - 2.*rstfac*cs2(T, eos)/3.)) / (rho*(cs2(T, eos) - u*u));
}

double dudr10(double r, double u, double T, double rho, t_eos eos)
{
    double Q, P, cs_2;

    Q = pow((rho/(eos.mu_i*mH)), 2) * cool(T, eos.metal);
    P = rho*kB*T/eos.mu_a/mH;
    cs_2 = cs2(T, eos);
    return ((eos.gamma-1.)*Q*r+2*eos.gamma*u*P) / (rho*r*(u*u - cs2(T, eos)));
}


double Mr(double r, double Rsc, double Msc)
{   
    if (r < Rsc) return Msc*(1. - pow(r/Rsc, 3));
    else return Msc;
}

double Vesc2(double r, double Rsc, double Msc, double MBH)
{
    //printf("Vesc = %12.5e\n", 2*G*(Mr(r, Rsc, Msc)+MBH)/(r+TINY));
    return 2*G*(Mr(r, Rsc, Msc)+MBH)/(r+TINY);
}

double eq_rho(double r, double u, double R_st, double qm, double Rsc, double dM)
{
    double rstfac;

    //printf("rho: %10.3e %10.3e %10.3e %10.3e %10.3e %10.3e\n", r, u, R_st, qm, Rsc, r/u);
    if (r < Rsc) {
        if (R_st == 0) rstfac = 1;
        else rstfac = (1-pow(R_st/r, 3));
        return qm*r/(3*u + TINY)*rstfac;
    } else {
        return dM/(4*pi*u*r*r + TINY);
    }
}

double dudr(t_solvec sol, double R_st, double T_st, t_clpar clpar, t_eos eos)
{
    double Q, rho, T, rstfac, qe, qm, cs;


    qe = get_qe(clpar.dE, clpar.Rsc, clpar.eta);
    qm = get_qm(clpar.dM, clpar.Rsc);
    T = sol.P*eos.mu_a*mH/kB/sol.rho;
    cs = sqrt(cs2(T, eos));
    Q = pow((sol.rho/(mH*eos.mu_i)), 2) * cool(T, eos.metal);
    //printf("dudr: %10.3e %10.3e %10.3e %10.3e\n", sol.rho, T, Q, rstfac);
    if (sol.u < cs) {
    //if (sol.r < clpar.Rsc) {
        if (R_st == 0) 
            rstfac = 1;
        else 
            rstfac = (1-pow(R_st/sol.r, 3));

        //printf("dudr1: %12.5e \n", ((eos.gamma-1.)*(qe-Q)+qm*(0.5*(eos.gamma+1.)*sol.u*sol.u - 2./3.*rstfac*(cs2(T, eos)
        //-0.25*Vesc2(sol.r, clpar.Rsc, clpar.Msc, clpar.MBH)))) / (sol.rho*(cs2(T, eos) - sol.u*sol.u)));

        return ((eos.gamma-1.)*(qe-Q)+qm*(0.5*(eos.gamma+1.)*sol.u*sol.u - 2./3.*rstfac*(cs2(T, eos)
        -0.25*Vesc2(sol.r, clpar.Rsc, clpar.Msc, clpar.MBH)))) / (sol.rho*(cs2(T, eos) - sol.u*sol.u));

    } else {
        //printf("dudr2: %12.5e\n", (2*sol.u/sol.r)*(2*pi*(eos.gamma-1.)*Q*sol.r*sol.r*sol.r/clpar.dM + cs2(T, eos) 
        //- 0.25*Vesc2(sol.r, clpar.Rsc, clpar.Msc, clpar.MBH)) / (sol.u*sol.u - cs2(T, eos)));
        return (2*sol.u/sol.r)*(2*pi*(eos.gamma-1.)*Q*sol.r*sol.r*sol.r/clpar.dM + cs2(T, eos) 
        - 0.25*Vesc2(sol.r, clpar.Rsc, clpar.Msc, clpar.MBH)) / (sol.u*sol.u - cs2(T, eos));
        
        

    }

}

double dPdr(t_solvec sol, double dudr, double R_st, t_clpar clpar, t_eos eos)
{
    double rho, Ve2, qm, T, cs;

    //printf("dPdr: r, u, rho = %10.3e %10.3e %10.3e\n", sol.r , sol.u, sol.rho);
    qm = get_qm(clpar.dM, clpar.Rsc);
    T = sol.P*eos.mu_a*mH/kB/sol.rho;
    cs = sqrt(cs2(T, eos));
    //if (sol.r < clpar.Rsc) {
    if (sol.u < cs) {
        //printf("dPdr1: %12.5e\n", -sol.rho*sol.u*dudr - qm*sol.u - G*sol.rho*(Mr(sol.r, clpar.Rsc, clpar.Msc)+clpar.MBH)/(sol.r*sol.r+TINY));
        return -sol.rho*sol.u*dudr - qm*sol.u - G*sol.rho*(Mr(sol.r, clpar.Rsc, clpar.Msc)+clpar.MBH)/(sol.r*sol.r+TINY);
    } else {
        Ve2 = Vesc2(sol.r, clpar.Rsc, clpar.Msc, clpar.MBH);
        //printf("dPdr2: %12.5e\n", -clpar.dM/(4*pi*sol.r*sol.r+TINY)*(dudr + Ve2/(2*sol.r*sol.u+TINY)));
        return -clpar.dM/(4*pi*sol.r*sol.r+TINY)*(dudr + Ve2/(2*sol.r*sol.u+TINY));
    }

}

void rk4step(double dr, t_solvec sol, double R_st, double T_st, t_clpar clpar, t_eos eos, t_solvec *nsol)
{
    double dudr1, dudr2, dudr3, dudr4, dPdr1, dPdr2, dPdr3, dPdr4;
    double qm;
    t_solvec sol1, sol2, sol3;

    qm = get_qm(clpar.dM, clpar.Rsc);
    //printf("rk4s: %10.3e %10.3e %10.3e %10.3e %10.3e %10.3e %10.3e\n", dr, r, u, P, R_st, dM, Rsc);
    //printf("rk4c: %10.3e %10.3e %10.3e %10.3e %10.3e %10.3e %10.3e\n", qe, qm, Rsc, Msc, MBH, gamma, mu_a);
    //printf("rk4d: %10.3e %10.3e\n", mu_i, metal);
    dudr1 = dudr(sol, R_st, T_st, clpar, eos);
    dPdr1 = dPdr(sol, dudr1, R_st, clpar, eos);
    sol1.r = sol.r + 0.5*dr;
    sol1.u = sol.u + 0.5*dr*dudr1;
    sol1.P = sol.P + 0.5*dr*dPdr1;
    sol1.rho = eq_rho(sol1.r, sol1.u, R_st, qm, clpar.Rsc, clpar.dM);
    //printf("sol1: %10.3e %10.3e %10.3e %10.3e\n", sol1.r, sol1.rho, sol1.u, sol1.P);

    dudr2 = dudr(sol1, R_st, T_st, clpar, eos);
    dPdr2 = dPdr(sol1, dudr1, R_st, clpar, eos);
    sol2.r = sol.r + 0.5*dr;
    sol2.u = sol.u + 0.5*dr*dudr2;
    sol2.P = sol.P + 0.5*dr*dPdr2;
    sol2.rho = eq_rho(sol1.r, sol1.u, R_st, qm, clpar.Rsc, clpar.dM);
    //printf("sol2: %10.3e %10.3e %10.3e %10.3e\n", sol2.r, sol2.rho, sol2.u, sol2.P);

    dudr3 = dudr(sol2, R_st, T_st, clpar, eos);
    dPdr3 = dPdr(sol2, dudr1, R_st, clpar, eos);
    sol3.r = sol.r + dr;
    sol3.u = sol.u + dr*dudr3;
    sol3.P = sol.P + dr*dPdr3;
    sol3.rho = eq_rho(sol1.r, sol1.u, R_st, qm, clpar.Rsc, clpar.dM);
    //printf("sol3: %10.3e %10.3e %10.3e %10.3e\n", sol3.r, sol3.rho, sol3.u, sol3.P);

    dudr4 = dudr(sol3, R_st, T_st, clpar, eos);
    dPdr4 = dPdr(sol3, dudr1, R_st, clpar, eos);
    //printf("sol4: %10.3e %10.3e %10.3e %10.3e\n", sol2.r, sol2.rho, sol2.u, sol2.P);

    (*nsol).r = sol.r + dr;
    (*nsol).u = sol.u + dr*(dudr1/6. + dudr2/3. + dudr3/3. + dudr4/6.);
    (*nsol).P = sol.P + dr*(dPdr1/6. + dPdr2/3. + dPdr3/3. + dPdr4/6.);
    (*nsol).rho = eq_rho((*nsol).r, (*nsol).u, R_st, qm, clpar.Rsc, clpar.dM);

    //printf("rk4e: %10.3e %10.3e %10.3e %10.3e %10.3e %10.3e %10.3e\n", *unew, *Pnew, du1, dP1, du4, dP4);

}

t_solvec *rk4_integrate(double T_st, double R_st, t_clpar clpar, t_eos eos, t_numpar numpar, double *Rson_test)
{
    double dr, T, qe, qm, cs;
    double dudr_p, dPdr_p, dudr7_p, dPdr7_p;
    double delta;
    double rsav, drsav;
    t_solvec *solution, sol, sol1, sol11, sol12;
    int i, steps = 0, steps_fail = 0, isav = 0;

    //printf("rk4i: %p, %10.3e, %10.3e, %10.3e\n", Rson_test, *Rson_test, numpar.Nrmin, numpar.Nrmax);
    //printf("rk4 called: %15.8e %15.8e %15.8e\n", T_st, R_st, *Rson_test);
    

    //printf("rk4 start\n");
    if (*Rson_test) *Rson_test = INFINITY;

    // initial step & solution vector
    dr = clpar.Rsc / numpar.Nrmin;
    drsav = numpar.Rmax / numpar.Nrmin;
    qe = get_qe(clpar.dE, clpar.Rsc, clpar.eta);
    qm = get_qm(clpar.dM, clpar.Rsc);
    sol.r = R_st;
    sol.rho = eos.mu_i*mH*n_st(T_st, qe, qm, eos);
    sol.u = 0;
    sol.P = sol.rho*kB*T_st/eos.mu_a/mH;
    //printf("rk4 called: qe, qm, rho = %12.5e %12.5e %12.5e\n", qe, qm, sol.rho);

    // check intial conditions
    if (sol.rho < 0) {
        *Rson_test = -1;
        return NULL;
    }
    //if (*Rson_test == 0)
    //printf("%10.3e %10.3e %10.3e %10.3e %10.3e %10.3e %10.3e\n"
    //, sol.r, sol.u, sol.P, T_st, sol.rho, sqrt(cs2(T_st, eos)), numpar.Rmax);

    if (!(*Rson_test)) {
        solution = (t_solvec *)malloc((numpar.Nrmin+1)*sizeof(t_solvec));
        for (i=0; i <= numpar.Nrmin; i++) {solution[i].r = -1;}
        solution[0] = sol;
    }

    rsav = R_st;
    isav++;


    while (sol.r < numpar.Rmax) {
        if ((sol.r < clpar.Rsc) && ((sol.r + dr) > clpar.Rsc)) {
            sol.r = sol.r+dr;
        }
        rk4step(dr, sol, R_st, T_st, clpar, eos, &sol1);
        //printf(" 1: %10.3e %10.3e %10.3e %10.3e %10.3e\n", dr, sol1.r, sol1.rho, sol1.u, sol1.P);

        rk4step(0.5*dr, sol,   R_st, T_st, clpar, eos, &sol11);
        rk4step(0.5*dr, sol11, R_st, T_st, clpar, eos, &sol12);
        //printf(" 2: %10.3e %10.3e %10.3e %10.3e %10.3e\n", dr, sol12.r, sol12.rho, sol12.u, sol12.P);

        // compute accuracy
        delta = fabs(sol12.u - sol1.u)/fabs(sol12.u+sol1.u);
        steps++;

        // check if the maximum step number is not exceeded
        if (steps > NMAXSTEPS) {
            if (!(*Rson_test)) {
                *Rson_test = -2;
                solution[isav] = sol;
                return solution;

            } else {
                *Rson_test = INFINITY;
                return NULL;
            }
        }

        // calculate some quantities used in solution diagnosis
        T = sol.P*eos.mu_a*mH/kB/sol.rho;
        cs = sqrt(cs2(T, eos));
        dudr_p = (sol12.u-sol.u)/dr;
        dPdr_p = (sol12.P-sol.P)/dr;

        // print solution point
        //if (!(*Rson_test)) {
        //    printf("%10.3e %20.10e %15.8e %10.3e %10.3e %10.3e %10.3e %10.3e %10.3e %10.3e\n"
        //    , dr, sol.r, sol.u, sol.P, T, sol.rho, cs, dudr_p, dPdr_p, delta);
        //}

        // sonic point found
        if (*Rson_test && sol.u > cs) {
            //solution[isav] = sol;
            *Rson_test = sol.r;
            isav++;
            rsav = sol.r;
            //printf("sonic point, sol.r, Rson = %10.3e, %10.3e\n", sol.r, *Rson_test);
            break;
        }
        

        // step control
        if ((sol12.u < 0) || (sol12.P < 0) || (sol12.rho < 0) || (dudr_p < 0)) {
            // negative u, decrease dr by one order of mag
            dr /= 10.;
            steps_fail++;
        } else {
            if ((delta > numpar.rk4tol) && (dr > clpar.Rsc/numpar.Nrmax)) {
            //if ((delta > rk4tol)) {
                // insufficient accuracy; decrease step and repeat the step
                steps_fail++;
                dr = MAX(clpar.Rsc/numpar.Nrmax, MIN(clpar.Rsc/numpar.Nrmin, dr*0.99*pow(numpar.rk4tol/delta, 0.25)));
                //dr *= 0.99*pow(rk4tol/delta, 0.25);
                //dr = MIN(clpar.Rsc/Nr, dr*0.99*pow(rk4tol/delta, 0.25));
            } else {
                // solution accepted
                sol = sol12;
                dr = MAX(clpar.Rsc/numpar.Nrmax, MIN(clpar.Rsc/numpar.Nrmin, dr*0.99*pow(numpar.rk4tol/delta, 0.2)));
                //dr = MIN(clpar.Rsc/Nr, dr*0.99*pow(rk4tol/delta, 0.2));
                //dr = dr*0.99*pow(rk4tol/delta, 0.2);

                // save the data point
                if (!(*Rson_test) && ((sol.r - rsav) > drsav)) {
                    cs = sqrt(cs2(T, eos));
                    //printf("point saved: %4d %10.3e %10.3e %10.3e\n", isav, sol.r, sol.u, cs);
                    solution[isav] = sol;
                    isav++;
                    rsav = sol.r;
                }
            }
        }

    }
    //printf("rk4: proper end; steps = %d\n", steps);
    return solution;

}


double Rsonic(double T_st, double R_st, t_clpar clpar, t_eos eos, t_numpar numpar)
{
    double cs2c, u2, T , qe, qm;
    double Rson, uh, ul, csh, csl, q;
    int i, ind = -1;
    t_solvec *solution;
    t_numpar np;

    qe = get_qe(clpar.dE, clpar.Rsc, clpar.eta);
    qm = get_qm(clpar.dM, clpar.Rsc);
    //printf("Rson called: Tst, nst = %12.5e, %12.5e\n", T_st, n_st(T_st, qe, qm, eos));
    np = numpar;
    np.Rmax = 1.0*clpar.Rsc;
    Rson = 1.0;
    solution = rk4_integrate(T_st, R_st, clpar, eos, np, &Rson);
    return Rson;
}


double find_T_st(double R_st, double T_Pmax, t_clpar clpar, t_eos eos, t_numpar numpar)
{
    double Ta, Tb, Tc, Teps;
    double Ra, Rb, Rc, Rs;
    double qe, qm;
    
    qe = get_qe(clpar.dE, clpar.Rsc, clpar.eta);
    qm = get_qm(clpar.dM, clpar.Rsc);
    //printf("fTst called: %e %e %e\n", R_st, qe, qm);

    Ta = T_c_adia(qe, qm, eos);
    Teps = Ta / numpar.NT;
    //printf("Teps, NT, Ta = %12.5e, %12.5e, %12.5e\n", Teps, numpar.NT, Ta);
    Tb = T_Pmax;
    Rb = Rsonic(Tb, R_st, clpar, eos, numpar);
    //printf("Ta = %10.3e Tb = %10.3e, Rb = %10.3e, nst = %12.5e\n", Ta, Tb, Rb, n_st(Tb, qe, qm, eos));

    
    /*
    for (Tc = Ta; Tc > Tb; Tc *= 0.99) {
        Rs = Rsonic(Tc, R_st, clpar, eos, numpar);
        printf("Tst-ser: % 15.8e % 15.8e % 15.8e\n", Tc, Rs, clpar.Rsc);
    }
    */
    
    
    if ((Rb < clpar.Rsc)) {
        //printf("#central temperature not found\n");
        //printf("# searching for Tc; Ta = %10.3e Tb = %10.3e\n", Ta, Tb);
        //printf("# Rsc = %10.3e Rb = %10.3e\n", clpar.Rsc, Rb);
        return -1;
    }

    //printf("# Ta Tb Tc Rs Rsc\n");
    while (fabs(Ta-Tb) > Teps) {
        Tc = 0.5*(Ta+Tb);
        Rs = Rsonic(Tc, R_st, clpar, eos, numpar);
        //printf("Tst: % 15.8e % 15.8e % 15.8e % 15.8e % 15.8e\n", Ta, Tb, Tc, Teps, Rs, clpar.Rsc);
        if (Rs > clpar.Rsc) 
            Tb = Tc;
        else 
            Ta = Tc;
    }
    //printf("Tst found: %e %e %e %e %e\n", qe, qm, Tc, T_Pmax, 2*abs(Tc-T_Pmax)/(Tc+T_Pmax));
    return Tc;
}
            
double find_R_st(double T_Pmax, t_clpar clpar, t_eos eos, t_numpar numpar)
{
    double Ra, Rb, Rc, Rsa, Rsb, Rsc;
    
    //printf("fRst called: %e %e %e %e %e %e %e %e %e\n", qe, qm, Rsc, mu_a, mu_i, metal, Rmax, Nr, NT);
    Ra = 0;
    Rb = clpar.Rsc*0.99999;
    Rsa = Rsonic(T_Pmax, Ra, clpar, eos, numpar);
    Rsb = Rsonic(T_Pmax, Rb, clpar, eos, numpar);
    //printf("fRst: Ta,Tb,Ra,Rb = %e %e %e %e\n", Rsa, Rsb, Ra, Rb);
    while (fabs(Ra-Rb) > clpar.Rsc/MIN(1e10, numpar.Nrmax)) {
        Rc = 0.5*(Ra+Rb);
        Rsc = Rsonic(T_Pmax, Rc, clpar, eos, numpar);
        //printf("Rc = %15.8e Rsc = %15.8e\n", Rc, Rsc);
        if (Rsc < clpar.Rsc)
            Ra = Rc;
        else
            Rb = Rc;
    }
    return Rc;
}   


double find_Lcrit_new(double T_Pmax, t_clpar clpar, t_eos eos, t_numpar numpar)
{
    double La, Lb, Lc, Ta, Tb, Tc;
    double Leps, Lc_guess, Lambda_st, cst2;
    int    fac;
    t_clpar clpara, clparb;


#ifdef DEBUG
    printf("fLcnew called: %e %e %e %e %e %e %e %e %e\n"
    , T_Pmax, clpar.Rsc, clpar.vinf, eos.mu_a, eos.mu_i, eos.metal, numpar.Rmax, numpar.Nrmin, numpar.NT);
#endif

    cst2 = cs2(T_Pmax, eos);
    Lambda_st = cool(T_Pmax, eos.metal);
    Lc_guess = (6.*(eos.gamma-1.)*pi*alpha*alpha*eos.mu_i*eos.mu_i*mH*mH*clpar.Rsc*clpar.eta*clpar.eta*clpar.vinf*clpar.vinf*clpar.vinf*clpar.vinf)
    /(clpar.eta*Lambda_st*(eos.gamma+1.)) * (0.5*clpar.vinf*clpar.vinf - cst2/(eos.gamma-1.));
#ifdef DEBUG
    printf("Lc_guess, Lambda_st, cst = %15.8e, %15.8e, %15.8e km/s\n", Lc_guess, Lambda_st, sqrt(cst2)/1e5);
#endif

    // bracket the solution
    Ta = Tb = 1;
    clpara = clparb = clpar;
    for (fac = 2; (Ta*Tb > 0) && (fac < 1e10); fac *= 2) {
        // low limit
        clpara.dE = Lc_guess/fac;
        clpara.dM = vinf2dM(clpara.dE, clpara.vinf, clpara.eta);
        Ta = find_T_st(0.0, T_Pmax, clpara, eos, numpar);
        // high limit
        clparb.dE = Lc_guess*fac;
        clparb.dM = vinf2dM(clparb.dE, clparb.vinf, clparb.eta);
        Tb = find_T_st(0.0, T_Pmax, clparb, eos, numpar);
#ifdef DEBUG
        printf("bracket: fac, La, Lb, Ta, Tb = %5d, %15.8e, %15.8e, %15.8e, %15.8e\n", fac, Lc_guess/fac, Lc_guess*fac, Ta, Tb);
#endif
        if (Ta*Tb < 0) break;
    }
    if (Ta*Tb > 0) {
        // root is not bracketed, exiting
        return -1;
    }
    

    Leps = Lc_guess/numpar.NT;
    La = Lc_guess/fac;
    Lb = Lc_guess*fac;
    clpara.dE = Lc_guess/fac;
    clpara.dM = vinf2dM(clpara.dE, clpara.vinf, clpara.eta);
    Ta = find_T_st(0.0, T_Pmax, clpara, eos, numpar);
    clparb.dE = Lc_guess*fac;
    clparb.dM = vinf2dM(clparb.dE, clparb.vinf, clparb.eta);
    Tb = find_T_st(0.0, T_Pmax, clparb, eos, numpar);
    //printf("bracketed: fac, La, Lb, Ta, Tb = %d %12.5e %12.5e %12.5e %12.5e\n", fac, La, Lb, Ta, Tb);

    /*
    for (Lc = La; Lc < Lb; Lc*= 1.01) {
        clpara.dE = Lc;
        clpara.dM = vinf2dM(clpara.dE, clpara.vinf, clpara.eta);
        Tc = find_T_st(0.0, T_Pmax, clpara, eos, Rmax, Nr, NT);
        printf("L-ser, Lc, Tc = %15.8e, %15.8e\n", Lc, Tc);
    }
    */

    // searching for Lcrit using bisection
    for (fac = 0; (fabs(La - Lb) > Leps) && (fac < 1000); fac++) {
        Lc = sqrt(La*Lb);
        clpara.dE = Lc;
        clpara.dM = vinf2dM(clpara.dE, clpara.vinf, clpara.eta);
        Tc = find_T_st(0.0, T_Pmax, clpara, eos, numpar);
#ifdef DEBUG
        printf("Labc, Tc = %15.8e %15.8e %15.8e, %15.8e\n", La, Lb, Lc, Tc);
#endif
        if (Tc < 0) {Lb = Lc; Tb = Tc;}
        else {La = Lc; Ta = Tc;}
    }
    
    if (fabs(La - Lb) > Leps) {
#ifdef DEBUG        
        printf("bisection didn't find Lcrit in 1000 steps\n");
#endif
        return -2; 
    } else {
#ifdef DEBUG        
        printf("Lcrit found: Lcrit, Lc_guess, fac = %15.8e, %15.8e, %d\n", Lc, Lc_guess, fac);
#endif
        return Lc;
    }
}

    
// READ ceinit {{{
void read_winit(t_clpar *clpar, t_eos *eos, t_numpar *numpar, char *cftype)
{
    FILE *f;
    char line[128], var[32];
    
    f = fopen("winit", "r");
    while (fscanf(f, "%[^\n]\n", line) != EOF) {
        //printf("READ: %s\n", line);
        sscanf(line, "%[^= ]=\n", var);
        if (!strcmp(var, "dE")) {
            sscanf(line, "%*[^=]=%lf\n", &(clpar->dE));
            //printf("SETTING: dE = %14.7e\n", clpar->dE);
        } else if (!strcmp(var, "vinf")) {
            sscanf(line, "%*[^=]=%lf\n", &(clpar->vinf));
            clpar->vinf *= 1e5;
            //printf("SETTING: vinf = %14.7e\n", clpar->vinf);
        } else if (!strcmp(var, "Rsc")) {
            sscanf(line, "%*[^=]=%lf\n", &(clpar->Rsc));
            clpar->Rsc *= cmpc;
            //printf("SETTING: Rsc = %14.7e\n", clpar->Rsc);
        } else if (!strcmp(var, "Rmax")) {
            sscanf(line, "%*[^=]=%lf\n", &(numpar->Rmax));
            numpar->Rmax *= cmpc;
            //printf("SETTING: Rmax = %14.7e\n", *Rmax);
        } else if (!strcmp(var, "Msc")) {
            sscanf(line, "%*[^=]=%lf\n", &(clpar->Msc));
            clpar->Msc *= MSun;
            //printf("SETTING: Msc = %14.7e\n", clpar->Msc);
        } else if (!strcmp(var, "MBH")) {
            sscanf(line, "%*[^=]=%lf\n", &(clpar->MBH));
            clpar->MBH *= MSun;
            //printf("SETTING: MBH = %14.7e\n", clpar->MBH);
        } else if (!strcmp(var, "dMld")) {
            sscanf(line, "%*[^=]=%lf\n", &(clpar->dMld));
            clpar->dMld *= MSun/syr;
            //printf("SETTING: dMld = %14.7e\n", clpar->dMld);
        } else if (!strcmp(var, "eta")) {
            sscanf(line, "%*[^=]=%lf\n", &(clpar->eta));
            //printf("SETTING: eta = %14.7e\n", clpar->eta);
        } else if (!strcmp(var, "gamma")) {
            sscanf(line, "%*[^=]=%lf\n", &(eos->gamma));
            //printf("SETTING: gamma = %14.7e\n", eos->gamma);
        } else if (!strcmp(var, "mu_a")) {
            sscanf(line, "%*[^=]=%lf\n", &(eos->mu_a));
            //printf("SETTING: mu_a = %14.7e\n", eos->mu_a);
        } else if (!strcmp(var, "mu_i")) {
            sscanf(line, "%*[^=]=%lf\n", &(eos->mu_i));
            //printf("SETTING: mu_i = %14.7e\n", eos->mu_i);
        } else if (!strcmp(var, "z")) {
            sscanf(line, "%*[^=]=%lf\n", &(eos->metal));
            //printf("SETTING: metal = %14.7e\n", eos->metal);
        } else if (!strcmp(var, "rk4tol")) {
            sscanf(line, "%*[^=]=%lf\n", &(numpar->rk4tol));
            //printf("SETTING: rk4tol = %14.7e\n", *rk4tol);
        } else if (!strcmp(var, "Nrmin")) {
            sscanf(line, "%*[^=]=%lf\n", &(numpar->Nrmin));
            //printf("SETTING: Nr = %14.7e\n", *Nr);
        } else if (!strcmp(var, "Nrmax")) {
            sscanf(line, "%*[^=]=%lf\n", &(numpar->Nrmax));
            //printf("SETTING: Nr = %14.7e\n", *Nr);
        } else if (!strcmp(var, "NT")) {
            sscanf(line, "%*[^=]=%lf\n", &(numpar->NT));
            //printf("SETTING: NT = %14.7e\n", *NT);
        } else if (!strcmp(var, "cftype")) {
            sscanf(line, "%*[^=]=%s\n", cftype);
            //printf("SETTING: cftype = %s\n", cftype);
        }
    }
    
    fclose(f);
}
// }}}


